<?xml version="1.0" encoding="utf-8"?>
<!-- $Revision: 1.1 $ -->

<!--

Things to do yet:

(*) Figures

(*) Bibliography, referenced by <bibref id="..." />

(*) Automatic dating based on build date

  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                version="1.0">

<xsl:output method="xml" indent="yes"/>

<xsl:template match="/">
    <fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">
	
        <fo:layout-master-set>
            <fo:simple-page-master master-name="cover"
                    page-height="11in"
                    page-width="8in">
                <fo:region-body margin-top="2in"
                                margin-left="1in" 
                                margin-right="0.8in"
                                margin-bottom="1.2in" 
                                border-top-style="solid" border-top-width="thin" border-top-color="silver" />
                <fo:region-before margin-top="1in"
                                  margin-left="1in"
                                  margin-bottom=".5in"
                                  extent="2in"
                                  border-bottom-style="solid" border-bottom-width="thin" border-bottom-color="silver" />
            </fo:simple-page-master>


            <fo:simple-page-master 
                master-name="page-basicPage"
                    page-height="29.7cm" 
                    page-width="21cm"
                    margin-top="1.5cm" 
                    margin-bottom="1.5cm" 
                    margin-left="1.5cm" 
                    margin-right="1.5cm">
                        <fo:region-before extent="1.2cm"/>
                        <fo:region-body margin-top="1.5cm" margin-bottom="1.5cm"/>
                        <fo:region-after extent="1.5cm"/>
            </fo:simple-page-master>

            <fo:page-sequence-master master-name="contents">
                <fo:repeatable-page-master-alternatives>
                    <fo:conditional-page-master-reference master-name="page-basicPage" odd-or-even="odd"/>
                    <fo:conditional-page-master-reference master-name="page-basicPage" odd-or-even="even"/>
                </fo:repeatable-page-master-alternatives>
            </fo:page-sequence-master>
        </fo:layout-master-set>
        
        <xsl:apply-templates select="whitePaper"/>
        
      </fo:root>
</xsl:template>

<xsl:template match="whitePaper">
        <!-- 
            The Cover Page 
        -->    
        <fo:page-sequence master-name="cover">
            <!--
                Paper header
            -->
            <fo:static-content flow-name="xsl-region-before">
                <fo:block text-align="center">	
                    <fo:table>
                        <fo:table-column column-width="4in"/>
                        <fo:table-column column-width="3in"/>
                        <fo:table-body>
                            <fo:table-row>
                                <fo:table-cell>
                                    <fo:block font-size="16pt" text-align="start">
                                        <fo:block>A JavaGeeks.com</fo:block>
                                        <fo:block>White Paper</fo:block>
                                    </fo:block>
                                </fo:table-cell>
                                <fo:table-cell>
                                    <fo:block font-size="8pt" text-align="start">
                                        <fo:block><xsl:value-of select="author/name" /></fo:block>
                                        <fo:block><xsl:value-of select="author/url" /></fo:block>
                                        <fo:block><xsl:value-of select="author/mail" /></fo:block>
                                        <fo:block><xsl:value-of select="author/date/day" /><xsl:value-of select="author/date/month" /> <xsl:value-of select="author/date/year" /></fo:block>
                                    </fo:block>
                                </fo:table-cell>
                            </fo:table-row>
                        </fo:table-body>
                    </fo:table>
                </fo:block>
            </fo:static-content>
            
            <!--
                Paper title, subtitle
            -->                
            <fo:flow flow-name="xsl-region-body">
                <fo:block> 
                    <fo:leader leader-pattern="rule" 
                               rule-thickness="6pt"
                               leader-length.optimum="100%"/> 
                </fo:block>
                <fo:block 
                        font-family="Helvetica" font-weight="bold" font-size="30pt" 
                        text-align="start"
                        line-height="36pt"
                        > 
                    <xsl:value-of select="title"/>
                </fo:block>
                <fo:block 
                        font-family="Helvetica" font-size="18pt"
                        line-height="24pt"
                        start-indent="30pt"> 
                    <xsl:text>&#xa0;</xsl:text>
                </fo:block>
                
                <fo:block 
                        start-indent="30pt">
                    <fo:leader leader-pattern="rule" 
                               rule-thickness="1pt"
                               leader-length.optimum="100%"/> 
                </fo:block>
                <fo:block 
                        font-family="Helvetica" font-size="18pt"
                        line-height="18pt"
                        start-indent="30pt"> 
                    <xsl:text>&#xa0;</xsl:text>
                </fo:block>
                <fo:block 
                        font-family="Helvetica" font-size="18pt"
                        line-height="24pt"
                        start-indent="30pt"> 
                    <xsl:value-of select="subtitle"/>
                </fo:block>
            </fo:flow>
            
        </fo:page-sequence> 

        <!--
            The content of the paper
        -->
        <fo:page-sequence master-name="contents" initial-page-number="2">
            <fo:static-content flow-name="xsl-region-before">
                <fo:block>
                    <fo:table>
                        <fo:table-column column-width="18cm" />
                        <fo:table-body>                            
                            <fo:table-row>                                                       
                                <fo:table-cell>
                                    <!-- Paper title & subtitle -->
                                    <fo:block text-align="start" font-size="12pt" font-family="serif" line-height="14pt" >
                                        <fo:inline font-weight="bold"><xsl:value-of select="/whitePaper/title"/></fo:inline>: <xsl:value-of select="/whitePaper/subtitle" />
                                    </fo:block>
                                </fo:table-cell>
                            </fo:table-row>                 
                        </fo:table-body>
                    </fo:table>  
                    <!-- dividing line -->
                    <fo:leader leader-pattern="rule" 
                               rule-thickness="1pt"
                               leader-length.optimum="100%"/> 
                </fo:block>                                                                                                                        
            </fo:static-content>
            
            <fo:static-content flow-name="xsl-region-after">
                <fo:block>
                    <!-- dividing line -->
                    <fo:leader leader-pattern="rule" rule-thickness="1pt" leader-length.optimum="100%"/>
                    <fo:table>
                        <fo:table-column column-width="6cm" />
                        <fo:table-column column-width="6cm" />
                        <fo:table-column column-width="6cm" />
                        <fo:table-body>                            
                            <fo:table-row>                                                       
                                <fo:table-cell>
                                    <fo:block text-align="start" font-size="12pt" font-family="serif" line-height="14pt">
                                        A Javageeks.com White Paper
                                    </fo:block>   
                                </fo:table-cell>
                                <fo:table-cell>
                                    <!-- Page -->
                                    <fo:block text-align="center" font-size="12pt" font-family="serif" font-weight="bold" line-height="14pt">
                                        Page <fo:page-number/>
                                    </fo:block>
                                </fo:table-cell>
                                <fo:table-cell>
                                    <fo:block text-align="end" font-size="12pt" font-family="serif" line-height="14pt">
                                        By <xsl:value-of select="/whitePaper/author/name" />
                                    </fo:block>   
                                </fo:table-cell>
                            </fo:table-row>                 
                        </fo:table-body>
                    </fo:table>  
                </fo:block>
            </fo:static-content>
            
            <fo:flow flow-name="xsl-region-body">
                
                <xsl:apply-templates select="abstract" />
                <xsl:apply-templates select="h1" />
                <xsl:apply-templates select="summary" />
                <xsl:call-template name="endnotes" />
                
                <xsl:if test="count(glossary/term) > 0">
                    <xsl:apply-templates select="glossary" />
                </xsl:if>
                
                <xsl:apply-templates select="biblio" />
                
                <xsl:call-template name="bigBlackBlock">
                    <xsl:with-param name="headingTitle">Copyright</xsl:with-param>
                </xsl:call-template>
                <fo:block font-family="Helvetica" font-size="8pt"
                        text-align="justify" 
                        start-indent="3em" end-indent="1em"
                        space-before="6pt" space-after="6pt" >
                    This paper, and accompanying source code, is copyright &#169; <xsl:value-of select="//author/date/year" /> by 
                    <xsl:value-of select="//whitePaper/author/name" />. All rights reserved. Usage for any other
                    purpose than personal or non-commercial education is expressly prohibited without
                    written consent. Code is copywrit under the Lesser GNU Public License (LGPL). For
                    questions or concerns, contact author.
                </fo:block>
            
                <xsl:call-template name="bigBlackBlock">
                    <xsl:with-param name="headingTitle">Colophon</xsl:with-param>
                </xsl:call-template>
                <fo:block font-family="Helvetica" font-size="8pt"
                        text-align="justify" 
                        start-indent="3em" end-indent="1em"
                        space-before="6pt" space-after="6pt" >
                    This paper was written using a standard text editor, captured
                    in a custom XML format, and rendered to PDF using the Apache
                    Xalan XSLT engine and the Apache FOP-0.17 XSL:FO engine. For
                    information on the process, contact the author at
                    <xsl:value-of select="//whitePaper/author/mail" />.
                    <xsl:if test="//whitePaper/attribute::revision != ''">
                    Revision <xsl:value-of select="//whitePaper/attribute::revision" />,
                    using whitePaper.xsl revision $Revision: 1.1 $.
                    </xsl:if>
                </fo:block>
            </fo:flow>
        </fo:page-sequence>
</xsl:template>

<!--
===============================================================================
                                                                       Sections
===============================================================================
-->
<xsl:template match="abstract">
    <xsl:call-template name="bigBlackBlock">
        <xsl:with-param name="headingTitle">Abstract</xsl:with-param>
    </xsl:call-template>
    
    <fo:block start-indent="3em" end-indent="1em">
        <xsl:apply-templates />
    </fo:block>    
</xsl:template>


<xsl:template match="h1">
    <xsl:call-template name="bigBlackBlock">
        <xsl:with-param name="headingTitle"><xsl:value-of select="@topic"/></xsl:with-param>
    </xsl:call-template>

    <fo:block start-indent="3em" end-indent="1em">
        <xsl:apply-templates />
    </fo:block>    
</xsl:template>


<xsl:template match="summary">
    <xsl:call-template name="bigBlackBlock">
        <xsl:with-param name="headingTitle">Summary</xsl:with-param>
    </xsl:call-template>
    
    <fo:block start-indent="3em" end-indent="1em">
        <xsl:apply-templates />
    </fo:block>    
</xsl:template>


<xsl:template match="biblio">
    <xsl:call-template name="bigBlackBlock">
        <xsl:with-param name="headingTitle">Bibliography</xsl:with-param>
    </xsl:call-template>

    <!-- Do an ordered list here -->
    <fo:list-block provisional-distance-between-starts="10mm" provisional-label-separation="1mm" space-before="6pt" space-after="6pt">
        <xsl:apply-templates select="child::*" /> 
    </fo:list-block>
    <!-- End ordered list -->
</xsl:template>

<xsl:template match="book">
    <fo:list-item space-after="1mm">
        <fo:list-item-label start-indent="5mm">
            <fo:block>&#x2022;</fo:block> 
        </fo:list-item-label>
        <fo:list-item-body>
            <fo:block>
                <fo:inline font-weight="bold"> [<xsl:value-of select="@id" />]</fo:inline>
                <fo:inline font-style="italic"><xsl:value-of select="title" /></fo:inline>, by <xsl:value-of select="author" />. <xsl:value-of select="publisher" /> (ISBN: <xsl:value-of select="isbn" />)
            </fo:block>
        </fo:list-item-body>
    </fo:list-item>  
  
  <!-- <fo:block>Book</fo:block> -->
</xsl:template>

<xsl:template match="periodical">
    <fo:list-item space-after="1mm">
        <fo:list-item-label start-indent="5mm">
            <fo:block>&#x2022;</fo:block>
        </fo:list-item-label>
        <fo:list-item-body>
            <fo:block>
                <fo:inline font-weight="bold"> [<xsl:value-of select="@id" />]</fo:inline>
                &#x22;<xsl:value-of select="title" />&#x22;, by <xsl:value-of select="author" />. 
                <fo:inline font-style="italic"><xsl:value-of select="journal" /></fo:inline> (<xsl:value-of select="date" />).
            </fo:block>
        </fo:list-item-body>
    </fo:list-item>
</xsl:template>

<xsl:template match="paper">
    <fo:list-item space-after="1mm">
        <fo:list-item-label start-indent="5mm">
            <fo:block>&#x2022;</fo:block>
        </fo:list-item-label>
        <fo:list-item-body>
            <fo:block>
                <fo:inline font-weight="bold"> [<xsl:value-of select="@id" />]</fo:inline>
                &#x22;<xsl:value-of select="title" />&#x22;, by <xsl:value-of select="author" /> (<xsl:value-of select="url" />).
            </fo:block>
        </fo:list-item-body>
    </fo:list-item>
</xsl:template>

<xsl:template match="glossary">
    <xsl:call-template name="bigBlackBlock">
        <xsl:with-param name="headingTitle">Glossary</xsl:with-param>
    </xsl:call-template>

    <fo:block>
        <fo:list-block provisional-distance-between-starts="10mm" provisional-label-separation="1mm" space-before="6pt" space-after="6pt">
            <xsl:for-each select="term">
                <fo:list-item space-after="1mm">
                    <fo:list-item-label start-indent="5mm">
                        <fo:block></fo:block>
                    </fo:list-item-label>
                    <fo:list-item-body>
                        <fo:block>
                            <fo:inline font-weight="bold"><xsl:value-of select="name" />. </fo:inline><xsl:value-of select="definition" />
                        </fo:block>
                    </fo:list-item-body>
                </fo:list-item>
            </xsl:for-each>
        </fo:list-block>
    </fo:block>
</xsl:template>

<!--
===============================================================================
                                                                Inline elements
===============================================================================
-->
<xsl:template match="p">
    <fo:block font-family="Helvetica" font-size="10pt" 
            text-align="justify" 
            space-before="6pt" space-after="6pt" >
        <fo:inline font-weight="bold">
            <xsl:value-of select="@topic" />
        </fo:inline>
        <xsl:text>&#x20;</xsl:text>
        <xsl:apply-templates />
    </fo:block>
</xsl:template>


<xsl:template match="em">
    <fo:inline font-style="italic">
        <xsl:apply-templates />
    </fo:inline>
</xsl:template>


<xsl:template match="codeFrag">
    <fo:block border-width="0.5pt" border-color="gray" text-align="start"
            padding-start="6pt" padding-end="6pt" white-space-collapse="false"
            space-before="6pt" space-after="6pt" >
            <!-- start-indent="" end-indent=""> -->
        <fo:inline font-family="Courier" font-size="8pt">
            <xsl:if test="@src != ''">
                ###<xsl:value-of select="@src" />###
                <!-- <xinc:include href="{@src}" parse="text" xmlns:xinc="http://www.w3.org/1999/XML/xinclude" /> -->
            </xsl:if>
            <xsl:if test="not(@src != '')" >
                <xsl:value-of select="." />
            </xsl:if>
        </fo:inline>
    </fo:block>
    <xsl:if test="@caption != ''">
        <fo:block text-align="center" space-after="6pt">
            <fo:inline font-family="Helvetica" font-size="8pt" font-weight="bold">
                <xsl:value-of select="@caption" />
            </fo:inline>
        </fo:block>
    </xsl:if>
</xsl:template>


<xsl:template match="figure">
	<fo:external-graphic>
    	<xsl:attribute name="src">file:<xsl:value-of select="@src"/></xsl:attribute>
    	<!-- <xsl:attribute name="width"><xsl:value-of select="@width"/>px</xsl:attribute> -->
    	<!-- <xsl:attribute name="height"><xsl:value-of select="@height"/>px</xsl:attribute> -->
	</fo:external-graphic>
    <xsl:if test="@caption != ''">
        <fo:block text-align="center" space-after="6pt">
            <fo:inline font-family="Helvetica" font-size="8pt" font-weight="bold">
                <xsl:value-of select="@caption" />
            </fo:inline>
        </fo:block>
    </xsl:if>
</xsl:template>


<xsl:template match="output">
    <fo:block background-color="black" color="white" 
            border-width="0.5pt" border-color="gray" text-align="start"
            padding-start="6pt" padding-end="6pt" white-space-collapse="false" 
            space-before="6pt" space-after="6pt" >
        <fo:inline font-family="Courier" font-size="8pt">
            <xsl:if test="@src != ''">
                ###<xsl:value-of select="@src" />###
                <!-- <xinc:include href="{@src}" parse="text" xmlns:xinc="http://www.w3.org/1999/XML/xinclude" /> -->
            </xsl:if>
            <xsl:if test="not(@src != '')" >
                <xsl:value-of select="." />
            </xsl:if>
        </fo:inline>
    </fo:block>
</xsl:template>


<xsl:template match="code">
    <fo:inline font-family="Courier">
        <xsl:value-of select="." />
    </fo:inline>
</xsl:template>


<xsl:template match="footnote">
    <fo:inline font-size="6pt" vertical-align="super"><xsl:number level="any" format="1"/></fo:inline>
<!--    
    <fo:footnote>
    	<fo:inline font-size="6pt" vertical-align="super"><xsl:number level="any" format="1"/></fo:inline>
    	<fo:footnote-body>
    	    <fo:block>
    	        <fo:inline font-size="6pt" vertical-align="super"><xsl:number level="any" format="1"/></fo:inline> <xsl:apply-templates />
    	    </fo:block>
    	</fo:footnote-body>
	</fo:footnote>    
-->    
</xsl:template>


<xsl:template match="quote">
    <fo:block background-color="lightgray" border-width="0.5pt" 
            space-before="6pt" space-after="6pt"
            start-indent="6em" end-indent="4em">
        <fo:block>
            <xsl:apply-templates />
        </fo:block>

        <xsl:if test="@from != ''">
            <fo:block font-style="italic" text-align="end" space-before="6pt" >
                --<xsl:value-of select="@from" />
            </fo:block>
        </xsl:if>    
    </fo:block>
</xsl:template>


<!-- This still needs major work -->
<xsl:template match="bibref">
    [<xsl:value-of select="@id" />]
</xsl:template>


<xsl:template match="ol">
    <fo:list-block provisional-distance-between-starts="6mm" provisional-label-separation="1mm" space-before="6pt" space-after="6pt">
        <xsl:apply-templates/>
    </fo:list-block>
</xsl:template>
<xsl:template match="ol/item">
    <fo:list-item space-after="1mm">
        <fo:list-item-label>
            <fo:block>
                <xsl:choose>
                    <xsl:when test="../@type != ''">
                        <xsl:number format="{../@type}"/>.
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:number format="1"/>.
                    </xsl:otherwise>
                </xsl:choose>
            </fo:block>
        </fo:list-item-label>
        <fo:list-item-body>
            <fo:block>
                <fo:inline font-style="italic"><xsl:value-of select="@title" /></fo:inline><xsl:text> </xsl:text><xsl:apply-templates/>
            </fo:block>
        </fo:list-item-body>
    </fo:list-item>
</xsl:template>


<xsl:template match="ul">
    <fo:list-block provisional-distance-between-starts="6mm" provisional-label-separation="1mm" space-before="6pt" space-after="6pt">
        <xsl:apply-templates/>
    </fo:list-block>
</xsl:template>
<xsl:template match="ul/item">
    <fo:list-item space-after="1mm">
        <fo:list-item-label>
            <xsl:choose>
                <xsl:when test="../@type ='disc'">
                    <fo:block>&#x2022;</fo:block>
                </xsl:when>
                <xsl:when test="../@type='square'">
                    <fo:block font-family="ZapfDingbats">&#110;</fo:block>
                </xsl:when>
                <xsl:when test="../@type='circle'">
                    <fo:block font-family="ZapfDingbats">&#109;</fo:block>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:choose>
                        <xsl:when test="count(ancestor::ul) = 1">
                            <fo:block>&#x2022;</fo:block>
                        </xsl:when>
                        <xsl:when test="count(ancestor::ul) = 2">
                            <fo:block font-family="ZapfDingbats">&#109;</fo:block>
                        </xsl:when>
                        <xsl:otherwise>
                            <fo:block font-family="ZapfDingbats">&#110;</fo:block>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:otherwise>
            </xsl:choose>
        </fo:list-item-label>
        <fo:list-item-body>
            <fo:block>
                <fo:inline font-style="italic"><xsl:value-of select="@title" /></fo:inline><xsl:text> </xsl:text><xsl:apply-templates/>
            </fo:block>
        </fo:list-item-body>
    </fo:list-item>
</xsl:template>

<!--
===============================================================================
                                                               Called templates
===============================================================================
-->
<xsl:template name="bigBlackBlock">
    <xsl:param name="headingTitle" />
    
    <fo:block font-family="Helvetica" font-size="12pt" font-weight="bold"
            background-color="black" color="white" 
            text-align="start"
            line-height="18pt"
            space-before="12pt" space-after="12pt" >
        <xsl:value-of select="$headingTitle" />
    </fo:block>
</xsl:template>

<xsl:template name="endnotes">
    <xsl:call-template name="bigBlackBlock">
        <xsl:with-param name="headingTitle">Notes</xsl:with-param>
    </xsl:call-template>
    
    <fo:block start-indent="3em" end-indent="1em"
            font-family="Helvetica" font-size="10pt" 
            text-align="justify" 
            space-before="6pt" space-after="6pt" >
        <xsl:for-each select="//footnote">
            <fo:block>
                [<xsl:number level="any" format="1"/>] <xsl:apply-templates />
            </fo:block>    
        </xsl:for-each>
    </fo:block>    

</xsl:template>

</xsl:stylesheet>
